@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2019 Hive Solutions Lda.
+# Copyright (c) 2008-2020 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2020 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -209,13 +209,12 @@ class Account(base.Base, authenticable.Authenticable):
         # creates the structure to be used as the root account description
         # using the default value and then stores the account as it's going
         # to be used as the default root entity (for administration)
+        password = appier.conf("ADMIN_PASSWORD", cls.ROOT_PASSWORD)
         account = cls(
             enabled = True,
             username = cls.ROOT_USERNAME,
             email = cls.ROOT_EMAIL,
-            password = cls.generate(
-                appier.conf("ADMIN_PASSWORD", cls.ROOT_PASSWORD)
-            ),
+            password = cls.generate(password),
             type = cls.ADMIN_TYPE
         )
         account.save(validate = False)
@@ -224,7 +223,7 @@ class Account(base.Base, authenticable.Authenticable):
         # uses it to print information about the newly created account
         account = cls.get(id = account.id, rules = False)
         logger.info("Username: %s" % account.username)
-        logger.info("Password: %s" % cls.ROOT_PASSWORD)
+        logger.info("Password: %s" % password)
         logger.info("Secret Key: %s" % account.key)
 
     @classmethod
@@ -247,6 +246,8 @@ class Account(base.Base, authenticable.Authenticable):
             appier.string_gt("password", 3),
             appier.string_lt("password", 256),
 
+            appier.not_duplicate("key", cls._name()),
+
             appier.not_null("type"),
 
             appier.not_empty("password_confirm"),
@@ -937,6 +938,16 @@ class Account(base.Base, authenticable.Authenticable):
             absolute = absolute
         )
 
+    @classmethod
+    @appier.link(name = "Duplicates Report JSON", context = True)
+    def duplicates_url(cls, view = None, context = None, absolute = False):
+        return appier.get_app().url_for(
+            "admin.duplicates_accounts_json",
+            view = view,
+            context = context,
+            absolute = absolute
+        )
+
     @property
     def confirmed(self):
         return self.enabled
